//
//  Play_Through_Controller.m
//  Play Through Lion
//
//  Created by John Geelen on 22-09-11.
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program Complex Play Through and Chen Kok's CocoaFilter.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	Cocoa, OSX10.6 <> 10.7 and XCODE 4.1.0
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

	#import "Play_Through_Controller.h"

	@implementation Play_Through_Controller

	#pragma mark -
	#pragma mark === Static Methods ===
//	*********************************************************************************************************************************
	static void	BuildDeviceMenu(AudioDeviceList *devlist, NSPopUpButton *menu, AudioDeviceID initSel);

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (void)awakeFromNib {
//	NSLog(@"apdAwakeFromNib");
	UInt32 propsize = 0;
	
	propsize = sizeof(AudioDeviceID);
	checkErr (AudioHardwareGetProperty(kAudioHardwarePropertyDefaultInputDevice, &propsize, &inputDevice));
	
	propsize = sizeof(AudioDeviceID);
	checkErr (AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice, &propsize, &outputDevice));
	
	BuildDeviceMenu(mInputDeviceList, mInputDevices, inputDevice);
	BuildDeviceMenu(mOutputDeviceList, mOutputDevices, outputDevice);
	
//	[box setFillColor:[NSColor colorWithCalibratedRed:0.95 green:1.0 blue:0.85 alpha:1.0]];
	[box setFillColor:[NSColor colorWithCalibratedRed:0.95 green:1.0 blue:0.85 alpha:1.0]];
}

- (id)init {
    self = [super init];
    if (self) {
        mInputDeviceList = new AudioDeviceList(true);
		mOutputDeviceList = new AudioDeviceList(false);
		playThru = 0;
    }
    
    return self;
}

- (void)dealloc {
	delete playThru;			
	playThru = 0;
	
	delete mInputDeviceList;
	delete mOutputDeviceList;
	
	[super dealloc];
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)inputDeviceSelected:(id)sender {
	NSLog(@"inputDeviceSelected");
	int val = [mInputDevices indexOfSelectedItem];
	
	if(playThru) {
		[self Reset:sender];
	}
	inputDevice =(mInputDeviceList->GetList())[val].mID;
}

- (IBAction)outputDeviceSelected:(id)sender {
	NSLog(@"outputDeviceSelected");
	int val = [mOutputDevices indexOfSelectedItem];
	
	if(playThru) {
		[self Reset:sender];
	}
	outputDevice =(mOutputDeviceList->GetList())[val].mID;
}

- (IBAction)StartStop:(id)sender {
	NSLog(@"apdStartStop");
	if(!playThru) {
		[self initPlaythru];
	}
	if(playThru) {
		if(!playThru->IsRunning()) {
			[mStartButton setTitle:@" Press to Stop"];
			playThru->Start();
			[mProgress startAnimation:sender];
		} else {	
			[mStartButton setTitle:@"Start Play Thru"];
			playThru->Stop();
			[mProgress stopAnimation:sender];
		}
	}	
}

- (IBAction)Reset:(id)sender {
	NSLog(@"Reset");
	if(playThru) {
		if(playThru->IsRunning()) {
			[self StartStop:sender];
		}
		delete playThru;
		playThru =0;
	}
}

#pragma mark -
#pragma mark === Audio Methodes ===
//	*********************************************************************************************************************************
- (void)initPlaythru {
	NSLog(@"apdInitPlaythru");
	playThru = new ComplexPlayThru(inputDevice,outputDevice);
	if(!playThru) {
		NSLog(@"ERROR: PlayThru init failed!");
		exit(1);
	}
}

	#pragma mark -
	#pragma mark === Static Methods ===
//	*********************************************************************************************************************************
static void	BuildDeviceMenu(AudioDeviceList *devlist, NSPopUpButton *menu, AudioDeviceID initSel) {
	NSLog(@"BuildDeviceMenu");
	
	[menu removeAllItems];
	
	AudioDeviceList::DeviceList &thelist = devlist->GetList();
	int index = 0;
	for (AudioDeviceList::DeviceList::iterator i = thelist.begin(); i != thelist.end(); ++i, ++index) {
		while([menu itemWithTitle:[NSString stringWithCString: (*i).mName]] != nil) {
			strcat((*i).mName, " ");
		}
		if([menu itemWithTitle:[NSString stringWithCString: (*i).mName]] == nil) {
			[menu insertItemWithTitle: [NSString stringWithCString: (*i).mName] atIndex:index];
			if (initSel == (*i).mID) {
				[menu selectItemAtIndex: index];
			}
		}
	}
}


@end
